

                   {UNIT...UNIT...UNIT...UNIT...UNIT}

   UNIT RVPARAMH;

    INTERFACE

     uses fonvect,fonscal,constype,jourjuli;

     {Cette procedure calcule les parametres d'une orbite connaissant
     la vitesse et le rayon vecteur que l on doit donner par leurs com-
     posantes dans le repere geocentrique i,j,k.

     Les resultats sont alors:

                param(1):grand axe en km.
                param(2):excentricite
     Les angles suivants en radian:
                param(3):inclinaison.
                param(4):petit omega,argument nodal du perigee
                param(5):grand omega,longitude du noeud
     L'anomalie excentrique est sans unit
                param(6):fi anomalie excentrique de la position
     La date de premier passage au perigee en jours juliens depuis
     le 01/01/2000.
                param(7):tp date de premier passage au perigee, en dcimal.

     On donnera la date DATIR et l heure HEURETIR sous forme
     STRING par exemple:

             3 mai 1986 :DATIR='03051986'
             9h6mn53s :HEURETIR='090653'.

     NB:Le programme fait les calculs en unites legales a cause des
     longueurs des vecteurs unitaires.Cela n empeche pas les sorties
     avec les unites classiques:km,km/s,radian.}


     PROCEDURE POSVITPARAMH(r,v:vecteur;datir,heuretir:string;mu:real;
                           var param:tabparam);

    IMPLEMENTATION

     procedure posvitparamh(r,v:vecteur;datir,heuretir:string;mu:real;
                           var param:tabparam);

      const i:vecteur=(1,0,0);
            j:vecteur=(0,1,0);
						k:vecteur=(0,0,1);

      var h,result,vect1,vect2,e,n:vecteur;
          fi,tp,mj,sj,jj2000,jjancien:real;
      BEGIN

       {Passage des longueurs en metres,des vitesses en m/s,mut en
       km3/s-2.}

       scalvect(1,r,r);
       scalvect(1,v,v);
			 mu:=mu*1;

			 {Calcul du moment cinetique}
			 prodvect(r,v,h);

			{Calcul du vecteur nodal}
			prodvect(k,h,n);

			{Calcul de l'excentricite}
			 prodvect(v,h,result);
			 scalvect (1/mu,result,vect1);
			 scalvect(-1/norm(r),r,vect2);
			 somme(vect1,vect2,e);
			 param[2]:=norm(e);

			 {Calcul de a elliptique}
                         if param[2]<1 then
			 param[1]:=-mu/(sqr(norm(v))-2*mu/norm(r));

                         {Calcul de a hyperbolique}
                         if param[2]>1 then
			 param[1]:=mu/(sqr(norm(v))-2*mu/norm(r));
                         if param[2]=1 then
                          Begin
                          writeLn('    Trajectoire parabolique, cas non prvu');
                          EXIT;
                          end;
			 {Calcul de l'inclinaison orbitale}
			 param[3]:=arccos(prodscal(h,k)/norm(h));
			 param[4]:=0;
			 param[5]:=0;
			 param[6]:=0;

			 {Cas d une orbite circulaire equatoriale}
			 if((param[2]<1e-6) and (abs(sin(param[3]))<1e-4))
				 then
						begin
							Writeln('Orbite circulaire equatoriale');
							param[2]:=0;
						 end;

			 {Cas de l orbite circulaire non equatoriale}
			 if ((param[2]<1e-6) and  (param[3]>=1e-4))
                         then
				 begin
					param[5]:=arccos(prodscal(n,i)/norm(n));
					if (prodscal(n,j)<0) then param[5]:=2*pi-param[5];
					writeln('Orbite circulaire non equatoriale');
					param[2]:=0;
				 end;

			 {Cas de l orbite equatoriale non circulaire}
			 if ((param[2]>=1e-6) and (abs(param[3])<1e-4))
			 then
				 begin
					param[5]:=0;
					param[4]:=arccos(prodscal(e,i)/norm(e));
					if (prodscal(e,j))<0 then param[4]:=2*pi-param[4];
					writeln('Orbite equatoriale non circulaire');
				 end;

			 {Cas de l orbite quelconque non equatoriale}
			 if ((param[2]>=1e-5) and (abs(sin(param[3]))>=1e-4))
			 then
				 begin
					 {Calcul de petit omega}
					 param[4]:=arccos(prodscal(n,e)/norm(n)/norm(e));
					 if(prodscal(e,k)<0) then param[4]:=2*pi-param[4];

					 {Calcul de grand omega}
					 param[5]:=arccos(prodscal(n,i)/norm(n));
					 if (prodscal(n,j)<0) then param[5]:=2*pi-param[5];
				 end;

                     {Calcul de tp et de fi cas ELLIPTIQUE}

                     if param[2]<1 then
                     BEGIN
			 fi:=0;

			 if param[2]>1e-6 then
			 begin
			   fi:=arccos((1-norm(r)/param[1])/param[2]);
			   if ((angvect(r,v)-pi/2)>=1e-5)
                            then fi:=2*pi-fi
			    else if (abs(angvect(r,v)-pi/2)<1e-5)
				 then if (norm(r)<param[1])
				      then fi:=0
				      else fi:=pi;
			 end;


			 {Calcul de la date julienne jj2000 en jours juliens}
			 datejuli(datir,heuretir,mj,sj,jj2000,jjancien);


			 tp:=jj2000+(exp(1.5*ln(param[1]))/sqrt(mu)*(2*pi-fi+
			 param[2]*sin(fi)))/86400;
			 param[6]:=fi;
			 param[7]:=tp;
			 param[1]:=param[1];


                      END;

                      {Calcul de tp et de fi cas HYPERBOLIQUE}

                     if param[2]>=1 then
                     BEGIN

			 fi:=argch((1+norm(r)/param[1])/param[2]);
			 if prodscal(r,V)<0  then fi:=-fi;



			 {Calcul de la date julienne jj2000 en jours juliens}
			 datejuli(datir,heuretir,mj,sj,jj2000,jjancien);


			 tp:=jj2000-(exp(1.5*ln(param[1]))/sqrt(mu)*(param[2]*sinh(fi)-fi))/86400;
			 param[6]:=fi;
			 param[7]:=tp;
			 param[1]:=param[1];

		     END;
         END;

   end.
